@@ -4,8 +4,8 @@
 ;; Maintainer: Jose A Ortega Ruiz <jao@gnu.org>
 ;; Keywords: docs, convenience
 ;; License: GPL-3.0-or-later
-;; Version: 0.7
-;; Package-Requires: ((emacs "26.1") (consult "0.18"))
+;; Version: 0.8
+;; Package-Requires: ((emacs "26.1") (consult "0.19"))
 ;; Homepage: https://codeberg.org/jao/consult-recoll
 
 ;; Copyright (C) 2021-2022  Free Software Foundation, Inc.
@@ -40,6 +40,8 @@
 
 (require 'seq)
 (require 'subr-x)
+(eval-when-compile (require 'cl-lib))
+
 (require 'consult)
 
 (declare-function 'eww-open-file "eww")
@@ -52,10 +54,21 @@
   "Prompt used by `consult-recoll'."
   :type 'string)
 
-(defcustom consult-recoll-search-flags '("-a")
-  "List of flags used to perform queries via recollq."
-  :type '(choice (const :tag "Query language" nil)
-                 (const :tag "All terms" ("-a"))
+(defcustom consult-recoll-search-flags 'query
+  "List of flags used to perform queries via recollq.
+
+The special values query and terms will select for you flags
+adequated to perform searches using recoll's query language or
+simply search for all provided terms, respectively.
+
+If you don't care about snippets and need speedier searches, you
+can also set this variable to the special values no-snippets or
+terms-no-snippets for query/all-terms searches that will return
+no snippets.."
+  :type '(choice (const :tag "Query language" query)
+                 (const :tag "All terms" terms)
+                 (const :tag "Query language sans snippets" no-snippets)
+                 (const :tag "All terms sans snippets" terms-no-snippets)
                  (list string)))
 
 (defcustom consult-recoll-open-fn nil
@@ -83,7 +96,7 @@ number), otherwise just with one."
 
 (defcustom consult-recoll-format-candidate nil
   "A function taking title, path and mime type, and formatting them for display.
-Set to nil to use the default 'title (path)' format."
+Set to nil to use the default `title (path)' format."
   :type '(choice (const nil) function))
 
 (defface consult-recoll-url-face '((t :inherit link))
@@ -98,12 +111,26 @@ Set to nil to use the default 'title (path)' format."
 (defvar consult-recoll-history nil "History for `consult-recoll'.")
 (defvar consult-recoll--current nil)
 (defvar consult-recoll--index 0)
+(defvar consult-recoll--snippets nil)
+
+(defun consult-recoll--search-flags ()
+  "Compute search flags according to `consult-recoll-search-flags'."
+  (cond ((listp consult-recoll-search-flags)
+         consult-recoll-search-flags)
+        ((symbolp consult-recoll-search-flags)
+         (cl-case consult-recoll-search-flags
+           (terms '("-A" "-p" "5" "-a"))
+           (no-snippets '())
+           (terms-no-snippets '("-a"))
+           (t '("-A" "-p" "5"))))
+        (t (user-error "Invalid value of `consult-recoll-search-flags'"))))
 
 (defun consult-recoll--command (text)
   "Command used to perform queries for TEXT."
   (setq consult-recoll--current nil)
   (setq consult-recoll--index 0)
-  `("recollq" "-A" "-p" "5" ,@consult-recoll-search-flags ,text))
+  (setq consult-recoll--snippets nil)
+  `("recollq" ,@(consult-recoll--search-flags) ,text))
 
 (defun consult-recoll--format (title urln mime)
   (if consult-recoll-format-candidate
@@ -130,10 +157,15 @@ Set to nil to use the default 'title (path)' format."
 (defsubst consult-recoll--candidate-index (candidate)
   (get-text-property 0 'index candidate))
 
-(defsubst consult-recoll--snippets (&optional candidate)
-  (get-text-property 0 'snippets (or candidate consult-recoll--current "")))
+(defsubst consult-recoll--snippets (candidate)
+  (let* ((len (length consult-recoll--snippets))
+         (idx (or (consult-recoll--candidate-index candidate) 0))
+         (pos (- len idx)))
+    (if (>= pos len)
+        ""
+      (mapconcat 'identity (reverse (elt consult-recoll--snippets pos)) "\n"))))
 
-(defun consult-recoll--search-snippet (candidate mime)
+(defun consult-recoll--search-snippet (candidate _mime)
   "When CANDIDATE is the text of a snippet, search for it in current buffer."
   (when (string-match "^\s+0 : " candidate)
     (let ((txt (replace-match "" nil nil candidate)))
@@ -148,6 +180,16 @@ Set to nil to use the default 'title (path)' format."
         (goto-char (match-beginning 0))
         (when (derived-mode-p 'org-mode) (org-reveal))))))
 
+(declare-function 'doc-view-goto-page "doc-view")
+(declare-function 'pdf-view-goto-page "pdf-view")
+
+(defun consult-recoll--open-file (filename &optional page)
+  "Default function for opening result files."
+  (find-file filename)
+  (when page
+    (cond ((derived-mode-p 'doc-view-mode) (doc-view-goto-page page))
+          ((derived-mode-p 'pdf-view-mode) (pdf-view-goto-page page)))))
+
 (defun consult-recoll--open (candidate)
   "Open file of corresponding completion CANDIDATE."
   (when candidate
@@ -157,7 +199,7 @@ Set to nil to use the default 'title (path)' format."
                        (consult-recoll-open-fn)
                        ((string= mime "text/html")
                         (lambda (f &optional _ignored) (eww-open-file f)))
-                       (t (lambda (f &optional _ignored) (find-file f))))))
+                       (t #'consult-recoll--open-file))))
       (if (not consult-recoll-inline-snippets)
           (funcall open url)
         (funcall open url (consult-recoll--candidate-page candidate))
@@ -185,14 +227,9 @@ Set to nil to use the default 'title (path)' format."
                                   'title title
                                   'index idx
                                   'size size)))
-           (prog1 (and (not (consult-recoll--snippets)) consult-recoll--current)
-             (setq consult-recoll--current cand))))
-        ((string= "/SNIPPETS" str)
-         (and (not consult-recoll-inline-snippets) consult-recoll--current))
-        ((string= "SNIPPETS" str)
-         (and consult-recoll-inline-snippets
-              (setq consult-recoll--current
-                    (propertize consult-recoll--current 'snippets t))))
+           (push () consult-recoll--snippets)
+           (setq consult-recoll--current cand)))
+        ((string-match-p "^/?SNIPPETS$" str) nil)
         ((and consult-recoll-inline-snippets consult-recoll--current)
          (when-let* ((page (and (string-match "^\\([0-9]+\\) :" str)
                                 (match-string 1 str)))
@@ -200,9 +237,7 @@ Set to nil to use the default 'title (path)' format."
                      (props (text-properties-at 0 consult-recoll--current)))
            (apply #'propertize (concat "    " str) 'page pageno props)))
         (consult-recoll--current
-         (let ((snippets (concat (consult-recoll--snippets) "\n" str)))
-           (setq consult-recoll--current
-                 (propertize consult-recoll--current 'snippets snippets)))
+         (push str (car consult-recoll--snippets))
          nil)))
 
 (defvar consult-recoll--preview-buffer "*consult-recoll preview*")
@@ -223,7 +258,8 @@ Set to nil to use the default 'title (path)' format."
              (insert (propertize url 'face 'consult-recoll-url-face) "\n")
              (insert (propertize (consult-recoll--candidate-mime candidate)
                                  'face 'consult-recoll-mime-face))
-             (when-let (s (consult-recoll--snippets candidate)) (insert "\n" s))
+             (when-let (s (consult-recoll--snippets candidate))
+               (insert "\n\n" s))
              (goto-char (point-min)))
            (pop-to-buffer buff)))
         ((eq action 'exit)
@@ -234,14 +270,19 @@ Set to nil to use the default 'title (path)' format."
   "If TRANSFORM return candidate, othewise extract mime-type."
   (if transform candidate (consult-recoll--candidate-mime candidate)))
 
+(defun consult-recoll--format-size (bytes)
+  "Format the given size with adaptive units."
+  (let ((szn (string-to-number bytes)))
+    (cond ((< szn 1024) (format "%s bytes" szn))
+          ((< szn 1048576) (format "%.1f Kbs" (/ szn 1024.0)))
+          ((< szn 1073741824) (format "%.1f Mbs" (/ szn 1024 1024)))
+          (t (format "%.1fs Gbs" (/ szn 1024 1024 1024))))))
+
 (defun consult-recoll--annotation (candidate)
   "Annotation for the given CANDIDATE (its size by default)"
-  (let* ((head (not (consult-recoll--candidate-page candidate)))
-         (size (consult-recoll--candidate-size candidate))
-         (mime (if head
-                   ""
-                 (format ", %s" (consult-recoll--candidate-mime candidate)))))
-    (format "     (%s bytes%s)" size mime)))
+  (and (not (consult-recoll--candidate-page candidate))
+       (format " (%s)" (consult-recoll--format-size
+                        (consult-recoll--candidate-size candidate)))))
 
 (defun consult-recoll--search (&optional initial)
   "Perform an asynchronous recoll search via `consult--read'.
